local _, addon = ...
local debug = addon.debug
local util = addon.utility

-- Function definitions in __debug/Debug.lua
-- Not included in release

function addon.debugPrint(...)
   local isDebugOn = util.GetDebugSetting()

   if isDebugOn then
      print(...)
   end
end

function addon.debugPrintTable(table, depth)
   local isDebugOn = util.GetDebugSetting()

   if isDebugOn then
      local space = " "
      local indentCount = 5

      if not depth then
         depth = 0
      end

      addon.debugPrint(format("%s{", string.rep(space, depth)))
      for k, v in pairs(table) do
         local indent = depth + indentCount
         if type(v) == "table" then
            addon.debugPrint(format("%s[%s] =", string.rep(space, indent), tostring(k)))
            addon.debugPrintTable(v, indent)
         else
            addon.debugPrint(format("%s[%s] = [%s]", string.rep(space, indent), tostring(k), tostring(v)))
         end
      end
      addon.debugPrint(format("%s}", string.rep(space, depth)))
   end
end

function addon.debugError(msg)
   --[===[@debug
   error(msg)
   --@end-debug]===]
end

function debug.debugContextMenu(menu, level)
   menu:AddOption  (  "Advance Time - Day",
         level,
         function(self)
            time.AdvanceDays(1)
            core.InitializePersistantVariables(false)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Advance Time - Week",
         level,
         function(self)
            time.AdvanceDays(7)
            core.InitializePersistantVariables(false)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Advance Time - Month",
         level,
         function(self)
            local dta = time.GetDaysToAdvance()
            local _, m, d, y, md = time.GetCalendarDate(dta)
            time.AdvanceDays(md - d + 1)
            core.InitializePersistantVariables(false)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Reset Time",
         level,
         function(self)
            time.ResetDays()
            core.InitializePersistantVariables(false)
            menu:CloseMenu()
         end
         )
   menu:AddDescriptor    (  "-------------------", level )
   menu:AddOption  (  "Earn 99 Gold",
         level,
         function(self)
            local cash = GetMoney()
            util.UpdateEarnedSpent(cash + (99 * GOLD), cash)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Spend 99 Gold",
         level,
         function(self)
            local cash = GetMoney()
            util.UpdateEarnedSpent(cash - (99 * GOLD), cash)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Earn 99 Silver",
         level,
         function(self)
            local cash = GetMoney()
            util.UpdateEarnedSpent(cash + (99 * SILVER), cash)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Spend 99 Silver",
         level,
         function(self)
            local cash = GetMoney()
            util.UpdateEarnedSpent(cash - (99 * SILVER), cash)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Earn 99 Copper",
         level,
         function(self)
            local cash = GetMoney()
            util.UpdateEarnedSpent(cash + (99 * COPPER), cash)
            menu:CloseMenu()
         end
         )
   menu:AddOption  (  "Spend 99 Copper",
         level,
         function(self)
            local cash = GetMoney()
            util.UpdateEarnedSpent(cash - (99 * COPPER), cash)
            menu:CloseMenu()
         end
         )
end
