local _, addon = ...
local slash = addon.slash

local minimap = addon.minimap
local util = addon.utility
local sd = addon.statDisplay
local test = addon.test

local CommandList = {}
local CommandListDescription = {}
local CommandListRaw = {}

local function ShowHelp()
   local c = _G.DEFAULT_CHAT_FRAME;
   c:AddMessage("|cffff6600".."WoW Money Statistics".."|r")
   c:AddMessage("|cffff6600".."Usage"..":|r  ".."/wms { "..string.lower(table.concat(CommandListRaw, " | ")).." | help }")
   for i=1, #CommandListRaw do
      local desc = CommandListDescription[CommandListRaw[i]]
      desc = desc and ":|r "..desc or "|r"
      c:AddMessage("  - |cffffd1b3"..string.lower(CommandListRaw[i])..desc)
   end
   c:AddMessage("  - |cffffd1b3".."help:|r Show Slash Command Usage")
end

local function ParseSlashCommand(msg)
   if #CommandListRaw == 0 then return end

   msg = (msg ~= "") and msg or "help"
   local cmd, args = string.match(msg, "([^%s]*)%s*(.*)")
   cmd = string.upper(cmd)

   if(CommandList[cmd]) then
      CommandList[cmd](args)
   else
      ShowHelp()
   end
end

local function RegisterSlashCommand(cmd, func, desc, hidden)
   if not hidden then
      hiddden = false
   end

   if(type(cmd) == "string" and not CommandList[string.upper(cmd)] and type(func) == "function") then
      CommandList[string.upper(cmd)] = func
      CommandListDescription[string.upper(cmd)] = desc

      if not hidden then
         table.insert(CommandListRaw, string.upper(cmd))
         table.sort(CommandListRaw)
      end
   end
end

SLASH_WOWMONEYSTATISTICS1 = '/wms'
SlashCmdList["WOWMONEYSTATISTICS"] = ParseSlashCommand

slash.RegisterSlashCommand = RegisterSlashCommand

RegisterSlashCommand("minimap",
                     function(...)
                        local isHidden = minimap.GetMinimapHideSetting()
                        _G.WOWMMChar.Minimap.IsMinimapHidden = minimap.SetMinimapHidden(not isHidden) 
                     end, 
                     "Toggle Minimap Icon")

if IsAddOnLoaded("SexyMap") then
   RegisterSlashCommand("sexymap",
                        function(...)
                           local isSexyCompat = util.GetSexyCompatSetting()
                           _G.WOWMMChar.Minimap.IsSexyCompat = (not isSexyCompat)
                           ReloadUI()
                        end, 
                        "Toggle SexyMap Integration")
end

RegisterSlashCommand("show",
                     function(...)
                        sd.ToggleDialog()
                     end,
                     "Toggle WMS Window")

RegisterSlashCommand("alt",
                     function(...)
                        local isAltIgnored = util.GetAltSetting()
                        _G.WOWMMGlobal.IsAltIgnored = (not isAltIgnored)

                        local c = _G.DEFAULT_CHAT_FRAME;
                        if isAltIgnored then
                           c:AddMessage("|cffff6600".."! - WMS - Alt Transactions Shown - !".."|r")
                        else
                           c:AddMessage("|cffff6600".."! - WMS - Alt Transactions Ignored - !".."|r")
                        end
                     end,
                     "Toggle Alt Transactions")

RegisterSlashCommand("verbose",
                     function(...)
                        local isVerbose = util.GetVerboseSetting()
                        _G.WOWMMChar.IsVerbose = (not isVerbose)

                        local c = _G.DEFAULT_CHAT_FRAME;
                        if isVerbose then
                           c:AddMessage("|cffff6600".."! - WMS - Transaction Messages Hidden - !".."|r")
                        else
                           c:AddMessage("|cffff6600".."! - WMS - Transaction Messages Shown - !".."|r")
                        end
                     end,
                     "Toggle Transaction Messages")

--[===[@test
RegisterSlashCommand("test",
                     function(...)
                        local args = {...}
                        test:RunTests(args[1])
                     end,
                     "Run Unit Testing")
--@end-test]===]

RegisterSlashCommand("debug",
                     function(...)
                        local isDebugOn = util.GetDebugSetting()
                        _G.WOWMMGlobal.IsDebugOn = (not isDebugOn)
                     end,
                     "",
                     true)
