local _, addon = ...

local mt = addon.mt
local util = addon.utility
local _G = _G

local player = UnitName("player")
local realm = GetRealmName()

local MERCH_UUID = 'MERCH_01'    -- Profits or Expenses from a Merchant
local REPAIR_UUID = 'MERCH_02'   -- Expenses from Repairing armor

local money
local repairCost
local bagValue
local UsedGB = false

function mt.GetBagsValue()
   local value = 0
   for i = 0, NUM_BAG_FRAMES do
      for j = 1, GetContainerNumSlots(i) do
         local count = select(2, GetContainerItemInfo(i, j))
         local hasNoValue, itemID = select(9, GetContainerItemInfo(i, j))
         if count and not hasNoValue then
            local sellPrice = select(11, GetItemInfo(itemID))
            value = value + (count * sellPrice)
         end
      end
   end

   return value
end

local function OnActivation(...)
   money = GetMoney()
   repairCost = GetRepairAllCost()
   bagValue = mt.GetBagsValue()
end

local function Classify(uuid)
   local rc, cr = GetRepairAllCost()
   local bv = mt.GetBagsValue()
   local m = GetMoney()
   local cost = m - money

   local TotalRepairCost = rc - repairCost
   local bagValueDelta = bagValue - bv
   local repaired = rc ~= repairCost

   if repaired and (cost == bagValueDelta) then
      -- If the Guild Bank picked up the tab no need for the addon to track it
      -- In this case the change in GetMoney is the same as what I would expect from
      -- the change in value of items in the users bags
      repaired = false
      bagValue = bv
      repairCost = rc
   end
   
   if uuid == REPAIR_UUID and CanMerchantRepair() then
      if repaired == true and TotalRepairCost ~= 0 then
         repairCost = rc
         repaired = false
         money = money + TotalRepairCost
         return REPAIR_UUID, TotalRepairCost
      end
   end

   if uuid == MERCH_UUID then
      if repaired and cost ~= 0 then
         cost = cost + TotalRepairCost
      end
      if cost ~= 0 then
         money = m
         return MERCH_UUID, cost
      end
   end
end

local function OnTrigger(uuid, t, m)
   if uuid == REPAIR_UUID then
      if m < 0 then
         m = math.abs(m)
         _G.WOWMSTracker[realm].AllChars[t].Spent = _G.WOWMSTracker[realm].AllChars[t].Spent + m
         _G.WOWMSTracker[realm][player][t].Spent = _G.WOWMSTracker[realm][player][t].Spent + m
      
         pcash = _G.WOWMMGlobal[realm].Chars[player].Cash
         local cash = pcash - m
         util.UpdateEarnedSpent(cash, pcash)
         util.UpdatePlayerCash(cash)
      else
         addon.debugError("ERROR - Repair that we earned money from?!")
      end
   else
      if m > 0 then
         _G.WOWMSTracker[realm].AllChars[t].Earned = _G.WOWMSTracker[realm].AllChars[t].Earned + m
         _G.WOWMSTracker[realm][player][t].Earned = _G.WOWMSTracker[realm][player][t].Earned + m
      
         pcash = _G.WOWMMGlobal[realm].Chars[player].Cash
         local cash = pcash + m
         util.UpdateEarnedSpent(cash, pcash)
         util.UpdatePlayerCash(cash)
      else
         m = math.abs(m)
         _G.WOWMSTracker[realm].AllChars[t].Spent = _G.WOWMSTracker[realm].AllChars[t].Spent + m
         _G.WOWMSTracker[realm][player][t].Spent = _G.WOWMSTracker[realm][player][t].Spent + m
      
         pcash = _G.WOWMMGlobal[realm].Chars[player].Cash
         local cash = pcash - m
         util.UpdateEarnedSpent(cash, pcash)
         util.UpdatePlayerCash(cash)
      end
   end
end

mt.RegisterTracker(MERCH_UUID, 'MERCH', 'MERCHANT_SHOW', 'MERCHANT_CLOSED', OnTrigger)
mt.RegisterTracker(REPAIR_UUID, 'REPAIR', 'MERCHANT_SHOW', 'MERCHANT_CLOSED', OnTrigger)
mt.RegisterOnActivation(MERCH_UUID, OnActivation)
mt.RegisterDeterminant(REPAIR_UUID, 'PLAYER_MONEY', Classify)
mt.RegisterDeterminant(MERCH_UUID, 'PLAYER_MONEY', Classify)

--TODO: Make a Grey / Green / Blue / Purple items catagory?
--TODO: Food / reagents / cooking supplies / ...
